@@ -10,7 +10,6 @@ try:
 except ImportError:
     import json
 
-
 ACTIVITY_GUID = int(uuid.uuid4().hex[0:8], 16)  
 
 class Activity(object):
@@ -24,8 +23,10 @@ class Activity(object):
 
     Valid constructors
 
-    >>> t = vipy.object.Track(category='Person').add(...))
-    >>> a = vipy.object.Activity(startframe=0, endframe=10, category='Walking', tracks={t.id():t})
+    ```python 
+    t = vipy.object.Track(category='Person').add(...))
+    a = vipy.object.Activity(startframe=0, endframe=10, category='Walking', tracks={t.id():t})
+    ```
 
     """
     def __init__(self, startframe, endframe, framerate=None, label=None, shortlabel=None, category=None, tracks=None, attributes=None, actorid=None, confidence=None, id=None):
@@ -48,7 +49,7 @@ class Activity(object):
             self._id = id  # use provided
         self._startframe = int(startframe)
         self._endframe = int(endframe)
-        self._framerate = framerate
+        self._framerate = float(framerate) if framerate is not None else framerate
         self._label = category if category is not None else label        
         self._shortlabel = self._label if shortlabel is None else shortlabel
         self._trackid = trackid
@@ -145,7 +146,7 @@ class Activity(object):
 
     def _set_framerate(self, fps):
         """Override framerate conversion and just set the framerate attribute.  This should really be set only in constructor.  Use with caution!"""
-        self._framerate = fps
+        self._framerate = float(fps)
         return self
 
     def framerate(self, fps=None, speed=None):
@@ -159,9 +160,9 @@ class Activity(object):
             assert fps is None or fps > 0, "Invalid framerate"
             assert self._framerate is not None, "Framerate conversion requires that the framerate is known for current activities.  This must be provided to the vipy.object.Activity() constructor."
 
-            fps = fps if fps is not None else (1.0/speed)*self._framerate            
+            fps = float(fps) if fps is not None else (1.0/speed)*self._framerate            
             (self._startframe, self._endframe) = [int(np.round(f*(fps/float(self._framerate)))) for f in (self._startframe, self._endframe)]
-            self._framerate = fps
+            self._framerate = float(fps)
             return self
     
     def category(self, label=None, shortlabel=None):
@@ -172,6 +173,29 @@ class Activity(object):
         else:
             return self._label
 
+    def categoryif(self, ifcategory, tocategory=None):
+        """If the current category is equal to ifcategory, then change it to newcategory.
+
+        Args:
+            
+            ifcategory [dict, str]: May be a dictionary {ifcategory:tocategory}, or just an ifcategory
+            tocategory [str]:  the target category 
+
+        Returns:
+        
+            this object with the category changed.
+
+        .. note:: This is useful for converting synonyms such as self.categoryif('person_sits', 'person_sitting')
+        """
+        assert (isinstance(ifcategory, dict) and tocategory is None) or tocategory is not None
+
+        if isinstance(ifcategory, dict):
+            for (k,v) in ifcategory.items():
+                self.categoryif(k, v)
+        elif self.category() == ifcategory:
+            self.category(tocategory, shortlabel=None)
+        return self
+
     def label(self, label=None, shortlabel=None):
         """Alias for category"""
         return self.category(label, shortlabel=shortlabel)
@@ -207,9 +231,10 @@ class Activity(object):
         return set(self._trackid)
 
     def hasoverlap(self, other, threshold=0):
-        assert isinstance(other, Activity), "Invalid input"
+        """Return true if the temporal_iou is greater than the provided threshold between self and other Track or other Activity"""
+        assert isinstance(other, Activity) or isinstance(other, Track), "Invalid input"
         assert threshold >= 0 and threshold <= 1, "Invalid temporal IOU threshold"
-        return (((min(self._endframe, other._endframe) - max(self._startframe, other._startframe)) > 0) if threshold == 0 else
+        return (((min(self._endframe, other.endframe()) - max(self._startframe, other.startframe())) > 0) if threshold == 0 else
                 self.temporal_iou(other) > threshold)
         
     def isneighbor(self, other, framegate=10):
@@ -222,10 +247,10 @@ class Activity(object):
         return trackid in self._trackid
 
     def hastrackoverlap(self, track):
-        """is the activity occurring during the interval when the track is occurring?"""
+        """is the activity occurring during the interval when the track is occurring and is this track assigned to the activity?"""
         assert isinstance(track, Track)
         return self.hastrack(track) and self.temporal_iou(track) > 0
-    
+
     def append(self, newtrack):
         """Append newtrack to this activity and set as actorid()"""
         assert isinstance(newtrack, Track), "Invalid input - must be vipy.object.Track"
