@@ -25,44 +25,60 @@ class Dataset():
     
     Common class to manipulate large sets of vipy objects in parallel
 
-    >>> D = vipy.dataset.Dataset([vipy.video.RandomScene(), vipy.video.RandomScene()], id='random_scene')
-    >>> with vipy.globals.parallel(2):
-    >>>     D = D.map(lambda v: v.frame(0))
-    >>> list(D)
+    ```python
+    D = vipy.dataset.Dataset([vipy.video.RandomScene(), vipy.video.RandomScene()], id='random_scene')
+    with vipy.globals.parallel(2):
+        D = D.map(lambda v: v.frame(0))
+    list(D)
+    ```
 
     Create dataset and export as a directory of json files 
 
-    >>> D = vipy.dataset.Dataset([vipy.video.RandomScene(), vipy.video.RandomScene()])
-    >>> D.tojsondir('/tmp/myjsondir')
+    ```python
+    D = vipy.dataset.Dataset([vipy.video.RandomScene(), vipy.video.RandomScene()])
+    D.tojsondir('/tmp/myjsondir')
+    ```
     
     Create dataset from all json or pkl files recursively discovered in a directory and lazy loaded
 
-    >>> D = vipy.dataset.Dataset('/tmp/myjsondir')  # lazy loading
+    ```python
+    D = vipy.dataset.Dataset('/tmp/myjsondir')  # lazy loading
+    ```
 
     Create dataset from a list of json or pkl files and lazy loaded
 
-    >>> D = vipy.dataset.Dataset(['/path/to/file1.json', '/path/to/file2.json'])  # lazy loading
+    ```python
+    D = vipy.dataset.Dataset(['/path/to/file1.json', '/path/to/file2.json'])  # lazy loading
+    ```
     
+    Args:
+    
+        - abspath [bool]: If true, load all lazy elements with absolute path
+        - loader [lambda]: a callable loader that will process the object .  This is useful for custom deerialization
+        - lazy [bool]: If true, load all pkl or json files using the custom loader when accessed
+
     .. notes:: Be warned that using the jsondir constructor will load elements on demand, but there are some methods that require loading the entire dataset into memory, and will happily try to do so
     """
 
-    def __init__(self, objlist, id=None, abspath=True):
+    def __init__(self, objlist, id=None, abspath=True, loader=None, lazy=False):
+
+        assert loader is None or callable(loader)
 
         self._saveas_ext = ['pkl', 'json']
         self._id = id if id is not None else vipy.util.shortuuid(8)
-        self._loader = lambda x: x
+        self._loader = (lambda x: x) if loader is None else loader
         self._istype_strict = True
-        self._lazy_loader = False
+        self._lazy_loader = lazy
         self._abspath = abspath
 
         if isinstance(objlist, str) and (vipy.util.isjsonfile(objlist) or vipy.util.ispklfile(objlist)):
             self._objlist = vipy.util.load(objlist, abspath=abspath)
         elif isinstance(objlist, str) and os.path.isdir(objlist):
-            self._objlist = vipy.util.listjson(objlist) + vipy.util.listpkl(objlist)
+            self._objlist = vipy.util.findjson(objlist) + vipy.util.findpkl(objlist)  # recursive
             self._loader = lambda x,b=abspath:  vipy.util.load(x, abspath=b) if (vipy.util.ispkl(x) or vipy.util.isjsonfile(x)) else x
             self._istype_strict = False
             self._lazy_loader = True
-        elif isinstance(objlist, list) and all([(vipy.util.ispkl(x) or vipy.util.isjsonfile(x)) for x in objlist]):
+        elif lazy and (isinstance(objlist, list) and all([(vipy.util.ispkl(x) or vipy.util.isjsonfile(x)) for x in objlist])):
             self._objlist = objlist 
             self._loader = lambda x,b=abspath:  vipy.util.load(x, abspath=b) if (vipy.util.ispkl(x) or vipy.util.isjsonfile(x)) else x            
             self._istype_strict = False
@@ -144,15 +160,26 @@ class Dataset():
         """Return True if all elements in the dataset are of type `vipy.video.Video`"""                
         return self.istype([vipy.video.Video])
 
-    def _is_vipy_scene(self):
+    def _is_vipy_video_scene(self):
         """Return True if all elements in the dataset are of type `vipy.video.Scene`"""                        
         return self.istype([vipy.video.Scene])
 
-    def clone(self):
-        """Return a deep copy of the dataset"""
-        return copy.deepcopy(self)
+    def _is_vipy_image_scene(self):
+        """Return True if all elements in the dataset are of type `vipy.video.Scene`"""                        
+        return self.istype([vipy.image.Scene])
 
-    def archive(self, tarfile, delprefix, mediadir='', format='json', castas=vipy.video.Scene, verbose=False, extrafiles=None, novideos=False, md5=True):
+    def clone(self, shallow=False):
+        """Return a deep copy of the dataset"""
+        if shallow:
+            objlist = self._objlist
+            self._objlist = []  
+            D = copy.deepcopy(self)
+            self._objlist = objlist  # restore
+            return D
+        else:
+            return copy.deepcopy(self)
+
+    def archive(self, tarfile, delprefix, mediadir='videos', format='json', castas=vipy.video.Scene, verbose=False, extrafiles=None, novideos=False, md5=True, tmpdir=None, inplace=False, bycategory=False, annotationdir='annotations'):
         """Create a archive file for this dataset.  This will be archived as:
 
            /path/to/tarfile.{tar.gz|.tgz|.bz2}
@@ -163,36 +190,52 @@ class Dataset():
                  extras1.ext
                  extras2.ext
         
-            Inputs:
-              - tarfile: /path/to/tarfilename.tar.gz
-              - delprefix:  the absolute file path contained in the media filenames to be removed.  If a video has a delprefix='/a/b' then videos with path /a/b/c/d.mp4' -> 'c/d.mp4', and {JSON|PKL} will be saved with relative paths to mediadir.  This may be a list of delprefixes.
-              - mediadir:  the subdirectory name of the media to be contained in the archive.  Usually "videos".             
-              - extrafiles: list of tuples or singletons [(abspath, filename_in_archive_relative_to_root), 'file_in_root_and_in_pwd', ...], 
-              - novideos [bool]:  generate a tarball without linking videos, just annotations
-              - md5 [bool]:  If True, generate the MD5 hash of the tarball using the system "md5sum", or if md5='vipy' use a slower python only md5 hash 
-              - castas [class]:  This should be a vipy class that the vipy objects should be cast to prior to archive.  This is useful for converting priveledged superclasses to a base class prior to export.
+            Args:
+                tarfile: /path/to/tarfilename.tar.gz
+                delprefix:  the absolute file path contained in the media filenames to be removed.  If a video has a delprefix='/a/b' then videos with path /a/b/c/d.mp4' -> 'c/d.mp4', and {JSON|PKL} will be saved with relative paths to mediadir.  This may be a list of delprefixes.
+                mediadir:  the subdirectory name of the media to be contained in the archive.  Usually "videos".             
+                extrafiles: list of tuples or singletons [(abspath, filename_in_archive_relative_to_root), 'file_in_root_and_in_pwd', ...], 
+                novideos [bool]:  generate a tarball without linking videos, just annotations
+                md5 [bool]:  If True, generate the MD5 hash of the tarball using the system "md5sum", or if md5='vipy' use a slower python only md5 hash 
+                castas [class]:  This should be a vipy class that the vipy objects should be cast to prior to archive.  This is useful for converting priveledged superclasses to a base class prior to export.
+                tmpdir:  The path to the temporary directory for construting this dataset.  Defaults to system temp
+                inplace [bool]:  If true, modify the dataset in place to prepare it for archive, else make a copy
+                bycategory [bool]: If true, save the annotations in an annotations/ directory by category
+                annotationdir [str]: The subdirectory name of annotations to be contained in the archive if bycategory=True.  Usually "annotations" or "json".
 
             Example:  
 
               - Input files contain /path/to/oldvideos/category/video.mp4
               - Output will contain relative paths videos/category/video.mp4
 
-              >>> d.archive('out.tar.gz', delprefix='/path/to/oldvideos', mediadir='videos')
+        ```python
+        d.archive('out.tar.gz', delprefix='/path/to/oldvideos', mediadir='videos')
+        ```
         
+            Returns:
+
+                The absolute path to the tarball 
         """
         assert self._isvipy(), "Source dataset must contain vipy objects for staging"
         assert all([os.path.isabs(v.filename()) for v in self]), "Input dataset must have only absolute media paths"
         assert len([v for v in self if any([d in v.filename() for d in tolist(delprefix)])]) == len(self), "all media objects must have a provided delprefix for relative path construction"
-        assert vipy.util.istgz(tarfile) or vipy.util.isbz2(tarfile), "Allowable extensions are .tar.gz, .tgz or .bz2"
+        assert vipy.util.istgz(tarfile) or vipy.util.isbz2(tarfile) or vipy.util.istar(tarfile), "Allowable extensions are .tar.gz, .tgz, .bz2 or .tar"
         assert shutil.which('tar') is not None, "tar not found on path"        
         
-        D = self.clone()
-        stagedir = remkdir(os.path.join(tempdir(), filefull(filetail(tarfile))))
+        D = self.clone() if not inplace else self   # large memory footprint if inplace=False
+        tmpdir = tempdir() if tmpdir is None else remkdir(tmpdir)
+        stagedir = remkdir(os.path.join(tmpdir, filefull(filetail(tarfile))))
         print('[vipy.dataset]: creating staging directory "%s"' % stagedir)
         delprefix = [[d for d in tolist(delprefix) if d in v.filename()][0] for v in self]  # select the delprefix per video
         D._objlist = [v.filename(v.filename().replace(os.path.normpath(p), os.path.normpath(os.path.join(stagedir, mediadir))), symlink=not novideos) for (p,v) in zip(delprefix, D.list())]
-        pklfile = os.path.join(stagedir, '%s.%s' % (filetail(filefull(tarfile)), format))
-        D.save(pklfile, relpath=True, nourl=True, sanitize=True, castas=castas, significant_digits=2, noemail=True, flush=True)
+
+        # Save annotations:  Split large datasets into annotations grouped by category to help speed up loading         
+        if bycategory:
+            for (c,V) in vipy.util.groupbyasdict(list(D), lambda v: v.category()).items():
+                Dataset(V, id=c).save(os.path.join(stagedir, annotationdir, '%s.%s' % (c, format)), relpath=True, nourl=True, sanitize=True, castas=castas, significant_digits=2, noemail=True, flush=True)
+        else:
+            pklfile = os.path.join(stagedir, '%s.%s' % (filetail(filefull(tarfile)), format))
+            D.save(pklfile, relpath=True, nourl=True, sanitize=True, castas=castas, significant_digits=2, noemail=True, flush=True)
     
         # Copy extras (symlinked) to staging directory
         if extrafiles is not None:
@@ -203,7 +246,7 @@ class Dataset():
                 os.symlink(os.path.abspath(e), os.path.join(stagedir, filetail(e) if a is None else a))
 
         # System command to run tar
-        cmd = ('tar %scvf %s -C %s --dereference %s %s' % ('j' if vipy.util.isbz2(tarfile) else 'z', 
+        cmd = ('tar %scvf %s -C %s --dereference %s %s' % ('j' if vipy.util.isbz2(tarfile) else ('z' if vipy.util.istgz(tarfile) else ''), 
                                                            tarfile,
                                                            filepath(stagedir),
                                                            filetail(stagedir),
@@ -227,15 +270,15 @@ class Dataset():
         """Save the dataset to the provided output filename stored as pkl or json
         
         Args:
-            outfile [str]: The /path/to/out.pkl or /path/to/out.json
-            nourl [bool]: If true, remove all URLs from the media (if present)
-            castas [type]:  Cast all media to the provided type.  This is useful for downcasting to `vipy.video.Scene` from superclasses
-            relpath [bool]: If true, define all file paths in objects relative to the /path/to in /path/to/out.json
-            sanitize [bool]:  If trye, call sanitize() on all objects to remove all private attributes with prepended '__' 
-            strict [bool]: Unused
-            significant_digits [int]: Assign the requested number of significant digits to all bounding boxes in all tracks.  This requires dataset of `vipy.video.Scene`
-            noemail [bool]: If true, scrub the attributes for emails and replace with a hash
-            flush [bool]:  If true, flush the object buffers prior to save
+            outfile: [str]: The /path/to/out.pkl or /path/to/out.json
+            nourl: [bool]: If true, remove all URLs from the media (if present)
+            castas: [type]:  Cast all media to the provided type.  This is useful for downcasting to `vipy.video.Scene` from superclasses
+            relpath: [bool]: If true, define all file paths in objects relative to the /path/to in /path/to/out.json
+            sanitize: [bool]:  If trye, call sanitize() on all objects to remove all private attributes with prepended '__' 
+            strict: [bool]: Unused
+            significant_digits: [int]: Assign the requested number of significant digits to all bounding boxes in all tracks.  This requires dataset of `vipy.video.Scene`
+            noemail: [bool]: If true, scrub the attributes for emails and replace with a hash
+            flush: [bool]:  If true, flush the object buffers prior to save
 
         Returns:        
             This dataset that is quivalent to vipy.dataset.Dataset('/path/to/outfile.json')
@@ -248,7 +291,7 @@ class Dataset():
             assert self._isvipy(), "Invalid input"
         if relpath:
             print('[vipy.dataset]: setting relative paths')
-            objlist = [v.relpath(filepath(outfile)) if os.path.isabs(v.filename()) else v for v in objlist]
+            objlist = [v.relpath(start=filepath(outfile)) if os.path.isabs(v.filename()) else v for v in objlist]
         if nourl: 
             print('[vipy.dataset]: removing URLs')
             objlist = [v.nourl() for v in objlist]           
@@ -260,7 +303,7 @@ class Dataset():
             print('[vipy.dataset]: casting as "%s"' % (str(castas)))
             objlist = [castas.cast(v) for v in objlist]                     
         if significant_digits is not None:
-            assert self._is_vipy_scene()
+            assert self._is_vipy_video_scene()
             assert isinstance(significant_digits, int) and significant_digits >= 1, "Invalid input"
             objlist = [o.trackmap(lambda t: t.significant_digits(significant_digits)) if o is not None else o for o in objlist]
         if noemail:
@@ -355,9 +398,11 @@ class Dataset():
     def merge(self, outdir):
         """Merge a dataset union into a single subdirectory with symlinked media ready to be archived.
 
-        >>> D1 = vipy.dataset.Dataset('/path1/dataset.json')
-        >>> D2 = vipy.dataset.Dataset('/path2/dataset.json')
-        >>> D3 = D1.union(D2).merge(outdir='/path3')
+        ```python
+        D1 = vipy.dataset.Dataset('/path1/dataset.json')
+        D2 = vipy.dataset.Dataset('/path2/dataset.json')
+        D3 = D1.union(D2).merge(outdir='/path3')
+        ```
 
         Media in D1 are in /path1, media in D2 are in /path2, media in D3 are all symlinked to /path3.
         We can now create a tarball for D3 with all of the media files in the same relative path.
@@ -399,10 +444,13 @@ class Dataset():
     
            Usage:
 
-           >>> D = vipy.dataset.Dataset(...).jsondir('/path/to/jsondir')
-           >>> D = vipy.util.load('/path/to/jsondir')   # recursively discover and lazy load all json files 
+        ```python
+        D = vipy.dataset.Dataset(...).jsondir('/path/to/jsondir')
+        D = vipy.util.load('/path/to/jsondir')   # recursively discover and lazy load all json files 
+        ```
 
            Args:
+
                outdir [str]:  The root directory to store the JSON files
                verbose [bool]: If True, print the save progress
                rekey [bool] If False, use the instance ID of the vipy object as the filename for the JSON file, otherwise assign a new UUID_dataset-index
@@ -433,19 +481,28 @@ class Dataset():
         return outdir
 
     def tojsondir(self, outdir=None, verbose=True, rekey=False, bycategory=False, byfilename=False, abspath=True):
-        """Alias for `vipy.dataset.jsondir`"""
+        """Alias for `vipy.dataset.Dataset.jsondir`"""
         return self.jsondir(outdir, verbose=verbose, rekey=rekey, bycategory=bycategory, byfilename=byfilename, abspath=abspath)
     
     def takelist(self, n, category=None, canload=False):
         """Take n elements of selected category and return list"""
         assert n >= 0, "Invalid length"
         D = self if category is None else self.clone().filter(lambda v: v.category() == category())
-        return [D[int(k)] for k in np.random.permutation(range(len(D)))][0:n]  # native python int
+        return [D[int(k)] for k in np.random.permutation(range(len(D)))[0:n]]  # native python int
+
+    def load(self):
+        """Load the entire dataset into memory.  This is useful for creating in-memory datasets from lazy load datasets"""
+        self._objlist = self.list()
+        self._loader = lambda x: x
+        return self
 
     def take(self, n, category=None, canload=False):
-        return Dataset(self.takelist(n, category=category, canload=canload))
+        D = self.clone(shallow=True)
+        D._objlist = self.takelist(n, category=category, canload=canload)
+        return D
 
     def take_per_category(self, n, id=None, canload=False):
+        D = self.clone()
         return Dataset([v for c in self.categories() for v in self.takelist(n, category=c, canload=canload)], id=id)
     
     def shuffle(self):
@@ -453,6 +510,11 @@ class Dataset():
         self._objlist.sort(key=lambda x: random.random())  # in place
         return self
 
+    def chunk(self, n):
+        """Yield n chunks of this dataset.  Last chunk will be ragged"""
+        for (k,V) in enumerate(vipy.util.chunklist(self._objlist, n)):
+            yield Dataset(V, id='%s_%d' % (self.id(), k), loader=self._loader)
+
     def split(self, trainfraction=0.9, valfraction=0.1, testfraction=0, seed=42):
         """Split the dataset by category by fraction so that video IDs are never in the same set"""
         assert self._isvipy(), "Invalid input"
@@ -460,11 +522,10 @@ class Dataset():
         assert valfraction >=0 and valfraction <= 1
         assert testfraction >=0 and testfraction <= 1
         assert trainfraction + valfraction + testfraction == 1.0
-
-        np.random.seed(seed)
-        A = self.list()
+        np.random.seed(seed)  # deterministic
         
         # Video ID assignment
+        A = self.list()
         videoid = list(set([a.videoid() for a in A]))
         np.random.shuffle(videoid)
         (testid, valid, trainid) = vipy.util.dividelist(videoid, (testfraction, valfraction, trainfraction))        
@@ -474,10 +535,11 @@ class Dataset():
                                        d['testset'] if 'testset' in d else [], 
                                        d['valset'] if 'valset' in d else [])
 
-        print('[vipy.dataset]: trainset=%d (%1.1f)' % (len(trainset), trainfraction))
-        print('[vipy.dataset]: valset=%d (%1.1f)' % (len(valset), valfraction))
-        print('[vipy.dataset]: testset=%d (%1.1f)' % (len(testset), testfraction))
-        
+        print('[vipy.dataset]: trainset=%d (%1.2f)' % (len(trainset), trainfraction))
+        print('[vipy.dataset]: valset=%d (%1.2f)' % (len(valset), valfraction))
+        print('[vipy.dataset]: testset=%d (%1.2f)' % (len(testset), testfraction))
+        np.random.seed()  # re-initialize seed
+
         return (Dataset(trainset, id='trainset'), Dataset(valset, id='valset'), Dataset(testset, id='testset') if len(testset)>0 else None)
 
     def tocsv(self, csvfile=None):
@@ -489,9 +551,11 @@ class Dataset():
 
         To perform this in parallel across four processes:
 
-        >>> D = vipy.dataset.Dataset(...)
-        >>> with vipy.globals.parallel(4):
-        >>>     D.map(lambda v: ...)
+        ```python
+        D = vipy.dataset.Dataset(...)
+        with vipy.globals.parallel(4):
+            D.map(lambda v: ...)
+        ```
 
         Args:
             f_map: [lambda] The lambda function to apply in parallel to all elements in the dataset.  This must return a JSON serializable object
@@ -503,7 +567,7 @@ class Dataset():
             ordered: [bool] If true, preserve the order of objects in dataset as returned from distributed processing
 
         Returns:
-            A `vipy.dataset.Dataset` containing the elements f_map(v).  This operation is order preserving.
+            A `vipy.dataset.Dataset` containing the elements f_map(v).  This operation is order preserving if ordered=True.
 
         .. note:: 
             - This dataset must contain vipy objects of types defined in `vipy.util.class_registry` or JSON serializable objects
@@ -518,9 +582,9 @@ class Dataset():
         from vipy.batch import Batch   # requires pip install vipy[all]
 
         # Distributed map using vipy.batch
-        f_serialize = lambda v,d=vipy.util.class_registry(): (str(type(v)), v.json()) if str(type(v)) in d else (None, json.dumps(v, ensure_ascii=False))  # fallback on JSON dumps/loads
+        f_serialize = lambda v,d=vipy.util.class_registry(): (str(type(v)), v.json()) if str(type(v)) in d else (None, pickle.dumps(v))  # fallback on PKL dumps/loads
         f_deserialize = lambda x,d=vipy.util.class_registry(): d[x[0]](x[1])  # with closure capture
-        f_catcher = vipy.util.catcher  # catch exceptions when executing lambda and return (True, result) or (False, exception)
+        f_catcher = lambda f, *args, **kwargs: vipy.util.loudcatcher(f, '[vipy.dataset.Dataset.map]: ', *args, **kwargs)  # catch exceptions when executing lambda, print errors and return (True, result) or (False, exception)
         S = [f_serialize(v) for v in self._objlist]  # local serialization
         B = Batch(vipy.util.chunklist(S, chunks), strict=strict, as_completed=ascompleted, warnme=False, minscatter=chunks, ordered=ordered)        
         if model is None:
@@ -529,10 +593,12 @@ class Dataset():
         else:
             f = lambda net, x, f_loader=self._loader, f_serializer=f_serialize, f_deserializer=f_deserialize, f_map=f_map, f_catcher=f_catcher: f_serializer(f_catcher(f_map, net, f_loader(f_deserializer(x))))  # with closure capture
             S = B.scattermap((lambda net, X, f=f: [f(net, x) for x in X]), model).result()  # chunked, scattered, caught exceptions
+        if not isinstance(S, list) or any([not isinstance(s, list) for s in S]):
+            raise ValueError('Distributed processing error - Batch returned: %s' % (str(S)))
         V = [f_deserialize(x) for s in S for x in s]  # Local deserialization and chunk flattening
         (good, bad) = ([r for (b,r) in V if b], [r for (b,r) in V if not b])  # catcher returns (True, result) or (False, exception string)
         if len(bad) > 0:
-            print('[vipy.dataset.Dataset.map]: Exceptions in map distributed processing:\n%s' % (len(bad), len(self), str(bad)))
+            print('[vipy.dataset.Dataset.map]: Exceptions in map distributed processing:\n%s' % str(bad))
             print('[vipy.dataset.Dataset.map]: %d/%d items failed' % (len(bad), len(self)))
         return Dataset(good, id=dst if dst is not None else id)
 
@@ -567,6 +633,17 @@ class Dataset():
     def frequency(self):
         return self.count()
 
+    def synonym(self, synonymdict):
+        """Convert all categories in the dataset using the provided synonym dictionary mapping"""
+        assert self._isvipy()
+        assert isinstance(synonymdict, dict)
+        
+        if self._is_vipy_video_scene():
+            return self.localmap(lambda v: v.trackmap(lambda t: t.categoryif(synonymdict)).activitymap(lambda a: a.categoryif(synonymdict)))
+        elif self._is_vipy_image_scene():
+            return self.localmap(lambda v: v.objectmap(lambda o: o.categoryif(synonymdict)))
+        return self
+
     def histogram(self, outfile=None, fontsize=6, category_to_barcolor=None, category_to_xlabel=None):
         assert self._isvipy()
         assert category_to_barcolor is None or all([c in category_to_barcolor for c in self.categories()])
@@ -627,11 +704,12 @@ class Dataset():
             vipy.metrics.histogram(d.values(), d.keys(), outfile=outfile, ylabel='Duration (frames)', fontsize=6)            
         return d
 
-    def duration_in_seconds(self, outfile=None, fontsize=6):
+    def duration_in_seconds(self, outfile=None, fontsize=6, max_duration=None):
         assert self._isvipy()
         d = {k:np.mean([v[1] for v in v]) for (k,v) in groupbyasdict([(a.category(), len(a)/v.framerate()) for v in self.list() for a in v.activitylist()], lambda x: x[0]).items()}
         if outfile is not None:
-            vipy.metrics.histogram(d.values(), d.keys(), outfile=outfile, ylabel='Duration (seconds)', fontsize=fontsize)            
+            max_duration = max(d.values()) if max_duration is None else max_duration
+            vipy.metrics.histogram([min(x, max_duration) for x in d.values()], d.keys(), outfile=outfile, ylabel='Duration (seconds)', fontsize=fontsize)            
         return d
 
     def framerate(self, outfile=None):
@@ -745,10 +823,10 @@ class Dataset():
         import vipy.torch    # lazy import, requires vipy[all] 
         from vipy.batch import Batch   # requires pip install vipy[all]
 
-        assert self._is_vipy_scene()
+        assert self._is_vipy_video_scene()
         outdir = vipy.util.remkdir(outdir)
         vipy.batch.Batch(self.list(), as_completed=True).map(lambda v, f=f_video_to_tensor, outdir=outdir, n_augmentations=n_augmentations: vipy.util.bz2pkl(os.path.join(outdir, '%s.pkl.bz2' % v.instanceid()), [f(v.print(sleep=sleep).clone()) for k in range(0, n_augmentations)]))
-        return vipy.torch.TorchTensordir(outdir)
+        return vipy.torch.Tensordir(outdir)
 
     def annotate(self, outdir, mindim=512):
         assert self._isvipy()
@@ -838,11 +916,13 @@ class Dataset():
     def zip(self, other, sortkey=None):
         """Zip two datasets.  Equivalent to zip(self, other).
 
-        >>> for (d1,d2) in D1.zip(D2, sortkey=lambda v: v.instanceid()):
-        >>>     pass
+        ```python
+        for (d1,d2) in D1.zip(D2, sortkey=lambda v: v.instanceid()):
+            pass
         
-        >>> for (d1, d2) in zip(D1, D2):
-        >>>     pass
+        for (d1, d2) in zip(D1, D2):
+            pass
+        ```
 
         Args:
             other: [`vipy.dataset.Dataset`] 
