@@ -25,7 +25,7 @@ except:
 def escape_to_exit(event):
     if event.key == 'escape' or event.key == 'q' or event.key == 'ctrl+c':
         import vipy.globals            
-        vipy.globals.user_hit_escape(True)
+        vipy.globals._user_hit_escape(True)
     
 def flush():
     plt.pause(0.001)
@@ -102,7 +102,7 @@ def _imshow_tight(img, fignum=None, keypress=True):
     if keypress:
         fig.canvas.mpl_connect('key_press_event', escape_to_exit)
         import vipy.globals    
-        vipy.globals.user_hit_escape(False)
+        vipy.globals._user_hit_escape(False)
 
     return (fig.number, imh)
 
@@ -131,7 +131,7 @@ def imshow(img, fignum=None):
     return fignum
 
 
-def text(caption, xmin, ymin, fignum=None, textcolor='black', textfacecolor='white', textfacealpha=1.0, fontsize=10, linewidth=3, facecolor='white', facealpha=0.5, alpha=1.0):
+def text(caption, xmin, ymin, fignum=None, textcolor='black', textfacecolor='white', textfacealpha=1.0, fontsize=10, linewidth=3, facecolor='white', facealpha=0.5, alpha=1.0, pad=0.5):
     plt.figure(fignum) if fignum is not None else plt.gcf()
     lw = linewidth  # pull in the boxes by linewidth so that they do not overhang the figure
 
@@ -139,9 +139,9 @@ def text(caption, xmin, ymin, fignum=None, textcolor='black', textfacecolor='whi
     captionoffset = 15*newlines   # move down a bit if near top of image, shift once per newline in caption    
     try:
         # MatplotlibDeprecationWarning: The 's' parameter of annotate() has been renamed 'text' since Matplotlib 3.3            
-        handle = plt.annotate(alpha=alpha, text=caption, xy=(xmin,ymin), xytext=(xmin, ymin+captionoffset), xycoords='data', color=textcolor, bbox=None if textfacecolor is None else dict(facecolor=textfacecolor, edgecolor=None, alpha=textfacealpha, boxstyle='square'), fontsize=fontsize, clip_on=True)
+        handle = plt.annotate(alpha=alpha, text=caption, xy=(xmin,ymin), xytext=(xmin, ymin+captionoffset), xycoords='data', color=textcolor, bbox=None if textfacecolor is None else dict(facecolor=textfacecolor, edgecolor=None, alpha=textfacealpha, boxstyle='square', pad=pad), fontsize=fontsize, clip_on=True)
     except:
-        handle = plt.annotate(alpha=alpha, s=caption, xy=(xmin,ymin), xytext=(xmin, ymin+captionoffset), xycoords='data', color=textcolor, bbox=None if textfacecolor is None else dict(facecolor=textfacecolor, edgecolor=None, alpha=textfacealpha, boxstyle='square'), fontsize=fontsize, clip_on=True)        
+        handle = plt.annotate(alpha=alpha, s=caption, xy=(xmin,ymin), xytext=(xmin, ymin+captionoffset), xycoords='data', color=textcolor, bbox=None if textfacecolor is None else dict(facecolor=textfacecolor, edgecolor=None, alpha=textfacealpha, boxstyle='square', pad=pad), fontsize=fontsize, clip_on=True)        
 
     return fignum
 
