@@ -33,9 +33,7 @@ try:
     import ujson as json  # faster
 except ImportError:
     import json
-
-
-
+    
 def class_registry():
     """Return a dictionary mapping str(type(obj)) to a JSON loader for all vipy objects.
 
@@ -69,7 +67,7 @@ def class_registry():
     except:
         registry.update( {"<class 'pycollector.admin.video.Video'>":lambda x: exec("raise ValueError(\"<class 'pycollector.admin.video.Video'> not found - This is for admin use only \")")})        
 
-    registry.update( {None:json.loads} )  # fallback on generic JSON dumps
+    registry.update( {None: cPickle.loads} )  # fallback on generic pickel dumps
     return registry
             
 
@@ -78,8 +76,10 @@ def save(vars, outfile=None):
 
     This function allows vipy objects to be serialized to disk for later loading.
 
-    >>> im = vipy.image.owl()
-    >>> im = vipy.util.load(vipy.util.save(im))   # round trip
+    ```python
+    im = vipy.image.owl()
+    im = vipy.util.load(vipy.util.save(im))   # round trip
+    ```
 
     Args:
         vars: A python object to save.  This can be any serializable python object
@@ -114,7 +114,7 @@ def save(vars, outfile=None):
     else:
         raise ValueError('Unknown file extension for save file "%s" - must be in %s' % (fileext(outfile), str(allowable)))
     
-    return outfile
+    return os.path.abspath(outfile)
 
 
 def load(infile, abspath=True, refcycle=True):
@@ -127,9 +127,11 @@ def load(infile, abspath=True, refcycle=True):
        3. if abspath=true, then convert relative paths to absolute paths for object when loaded
        4. If refcycle=False, then disable the python reference cycle garbage collector for large archive files
     
-       >>> im = vipy.image.owl()
-       >>> f = vipy.util.save(im)
-       >>> im = vipy.util.load(im)
+    ```python
+    im = vipy.image.owl()
+    f = vipy.util.save(im)
+    im = vipy.util.load(im)
+    ```
 
        Args:
            infile: [str] file saved using `vipy.util.save` with extension [.pkl, .json].  This may also be a directory tree containing json or pkl files 
@@ -218,14 +220,34 @@ def catcher(f, *args, **kwargs):
     except Exception as e:
         return (False, str(e))
 
+def loudcatcher(f, prepend, *args, **kwargs):
+    """Call the function f with the provided arguments, and return (True, result) on success and (False, exception) if there is any thrown exception.  Print the exception immediately.  Useful for parallel processing"""
+    assert callable(f)
+    try:
+        return (True, f(*args, **kwargs))
+    except Exception as e:
+        print('%s%s' % (prepend, str(e)))
+        return (False, str(e))
+
+
+def nonecatcher(f, *args, **kwargs):
+    """Call the function f with the provided arguments, and return (result) on success and (None) if there is any thrown exception.  Useful for parallel processing"""
+    assert callable(f)
+    try:
+        return f(*args, **kwargs)
+    except Exception as e:
+        return None
+
 
 def mergedict(d1, d2):
     """Combine keys of two dictionaries and return a dictionary deep copy.
     
-    >>> d1 = {1:2}
-    >>> d2 = {3:4}
-    >>> d3 = mergedict(d1,d2)
-    >>> assert d3 == {1:2, 3:4}
+    ```python
+    d1 = {1:2}
+    d2 = {3:4}
+    d3 = mergedict(d1,d2)
+    assert d3 == {1:2, 3:4}
+    ```
 
     """
     assert isinstance(d1, dict) and isinstance(d2, dict)
@@ -419,8 +441,10 @@ def dividelist(inlist, fractions):
 def chunklist(inlist, num_chunks):
     """Convert list into a list of lists of length num_chunks, such that each element is a list containing a sequential chunk of the original list.
     
-    >>> (A,B,C) = vipy.util.chunklist(inlist, num_chunks=3)
-    >>> assert len(A) == len(inlist) // 3
+    ```python
+    (A,B,C) = vipy.util.chunklist(inlist, num_chunks=3)
+    assert len(A) == len(inlist) // 3
+    ```
 
     .. note::  The last chunk will be larger for ragged chunks
     """
@@ -647,24 +671,54 @@ def repath(v, srcpath, dstpath):
     return vc
     
 
-def scpsave(V):
+def scpsave(V, username=None):
+    """Save an archive file to load via SCP.
+
+    Use case:
+
+    - This archive format is useful to allow access to videos and images that are accessible behind a remote server for which you have access via SSH key-based authentication.
+    - You create this archive on the remote server, and all vipy objects are replaced with references to remote media.
+    - Every video or image is replaced with a URL of the format 'scp://USER@HOST:/path/to.mp4'.  
+    - Vipy will use your SSH keys to SCP these media files from USER@HOST on demand, so that the videos are cached for you on your local machine when you need them.
+    - This is useful for transparently visualizing large datasets that are hidden behind an SSH-only accessible server
+
+    Usage:
+    
+    ```python
+    outfile = vipy.util.scpsave([vipy.video.Video(filename='/path/to.mp4)])  # run on remote machine that you have SSH key access
+    V = vipy.util.scpload(outfile)  # run on local machine that has SSH key access to remote machine
+    V[0].load()  # this will SCP the videos from 'scp:///path/to.mp4' to $VIPY_CACHE/to.mp4 transparently and on demand
+    ```
+
+    Args:
+        V: [vipy objects] A list of vipy objects or `vipy.dataset.Dataset`
+        username: [str] Your username on the remote machine to select the proper SSH key
+
+    Returns:
+        A temp archive file stored on the remote machine that will be downloaded and loaded via SCP
+
+    """
+    
     import vipy.image
     import vipy.video
-    if (isinstance(V, vipy.image.Image) or isinstance(V, vipy.video.Video)) and V.hasfilename():        
-        v = V
-        v = v.clone().url('scp://%s:%s' % (socket.gethostname(), v.filename())).nofilename()
+
+    if isinstance(V, vipy.dataset.Dataset) and V._isvipy():
+        v = V.localmap(lambda v: v.clone().url('scp://%s%s:%s' % (('%s@' % username) if username is not None else '', socket.gethostname(), v.filename())).nofilename())
+    elif (isinstance(V, vipy.image.Image) or isinstance(V, vipy.video.Video)) and V.hasfilename():        
+        v = V.clone().url('scp://%s%s:%s' % (('%s@' % username) if username is not None else '', socket.gethostname(), v.filename())).nofilename()
     elif islist(V) and all([isinstance(v, vipy.image.Image) or isinstance(v, vipy.video.Video) for v in V]):
-        v = [v.clone().url('scp://%s:%s' % (socket.gethostname(), v.abspath().filename())).nofilename() for v in V]
+        v = [v.clone().url('scp://%s%s:%s' % (('%s@' % username) if username is not None else '', socket.gethostname(), v.abspath().filename())).nofilename() for v in V]
     else:
         v = V # no vipy objects
 
-    pklfile = 'scp://%s:%s' % (socket.gethostname(), save(v, temppkl()))
+    pklfile = 'scp://%s%s:%s' % (('%s@' % username) if username is not None else '', socket.gethostname(), save(v, temppkl()))
     cmd = "V = vipy.util.scpload('%s')" % pklfile
     print('[vipy.util.scpsave]: On a remote machine where you have public key ssh access to this machine run:\n>>> %s\n' % cmd)
     return pklfile
 
 
 def scpload(url):
+    """Load an archive file saved using `vipy.util.scpsave`"""
     import vipy.downloader
     return load(vipy.downloader.scp(url, templike(url)))
 
@@ -759,9 +813,11 @@ def isinstalled(cmd):
 def isextension(filename, ext):
     """Does the filename end with the extension ext? 
     
-    >>> isextension('/path/to/myfile.json', 'json') == True
-    >>> isextension('/path/to/myfile.json', '.json') == True
-    >>> isextension('/path/to/myfile.json', '.pkl') == False
+    ```python
+    isextension('/path/to/myfile.json', 'json') == True
+    isextension('/path/to/myfile.json', '.json') == True
+    isextension('/path/to/myfile.json', '.pkl') == False
+    ```
 
     """
     return filename.endswith(ext)
@@ -1058,7 +1114,7 @@ def isS3url(path):
 
 def isyoutubeurl(path):
     """Is a path a youtube URL?"""
-    return isurl(path) and 'youtube.com' in path
+    return isurl(path) and ('youtube.com' in path or 'youtu.be' in path)
 
 def isRTSPurl(path):
     return isurl(path) and path.startswith('rtsp://')
@@ -1267,6 +1323,10 @@ def istgz(filename):
     """Is the filename a .tgz or .tar.gz extension?"""
     return filename[-4:] == '.tgz' or filename[-7:] == '.tar.gz'
 
+def istar(filename):
+    """Is the filename a .tar extension?"""
+    return filename[-4:] == '.tar'
+
 def isbz2(filename):
     """Is the filename a .bz2 or .tar.bz2 extension?"""
     return filename[-4:] == '.bz2' or filename[-8:] == '.tar.bz2'
@@ -1425,15 +1485,17 @@ class Stopwatch(object):
     
 class Timer(object):
     """Pretty print elapsed system time in seconds between calls to enter and exit
+    
+    ```python 
+       t = Timer():
+       [some code]
+       print(t)
+       [some more code]
+       print(t)
 
-       >>> t = Timer():
-       >>> [some code]
-       >>> print(t)
-       >>> [some more code]
-       >>> print(t)
-
-       >>> with Timer():
-       >>>    [some code]
+       with Timer():
+          [some code]
+    ```
        
     """
     def __enter__(self):
